<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

class WV_Documentation_Service {
	protected $cacheDir;

	public function __construct($cacheDir) {
		$this->cacheDir = sly_Util_Directory::create($cacheDir, null, true);
	}

	public function getDirectory() {
		return SLY_DEVELOPFOLDER.'/docs/';
	}

	public function getDocuments($withTitle = true) {
		$dir = new sly_Util_Directory($this->getDirectory());

		if (!$dir->exists()) {
			return array();
		}

		$files  = $dir->listPlain(true, false);
		$result = array();

		foreach ($files as $filename) {
			if (!preg_match('/\.textile$/', $filename, $match)) continue;
			$result[$filename] = $withTitle ? $this->getDocumentTitle($filename) : $filename;
		}

		return $result;
	}

	public function getDocumentTitle($filename) {
		$html = $this->getParsedDocument($filename);

		if (!preg_match('#<h1>(.*?)</h1>#i', $html, $match)) {
			return ucfirst(str_replace('.textile', '', $filename));
		}

		return strip_tags($match[1]);
	}

	public function getParsedDocument($filename, $asBackendHtml = false) {
		$realfile  = $this->getDirectory().$filename;
		$mtime     = filemtime($realfile);
		$encoded   = str_replace(DIRECTORY_SEPARATOR, '_', $filename);
		$suffix    = $asBackendHtml ? 'backend' : 'raw';
		$cacheFile = sprintf('%s/%s-%s.html', $this->cacheDir, $encoded, $suffix);

		if (!file_exists($cacheFile) || filemtime($cacheFile) < $mtime) {
			$html = $this->textile(file_get_contents($realfile));

			if ($asBackendHtml) {
				$html = $this->prepareForSally($html);
			}

			file_put_contents($cacheFile, $html);
		}
		else {
			$html = file_get_contents($cacheFile);
		}

		return $html;
	}

	protected function textile($text) {
		if (class_exists('GeSHi')) {
			preg_match_all('#(~{3,})([a-z0-9_]+)\1(.+?)\1#is', $text, $matches, PREG_SET_ORDER);

			foreach ($matches as $match) {
				$tildes   = $match[1];
				$language = $match[2];

				$geshi = new GeSHi(trim($match[3]), $language);
				$geshi->set_header_type(GESHI_HEADER_PRE);
				$geshi->enable_classes();
				$code = $geshi->parse_code();

				unset($geshi, $match[1], $match[2], $match[3]);

				// <pre>-Tag wieder entfernen

				$code = preg_replace('#^<pre.*?>#', '', $code);
				$code = preg_replace('#</pre>$#', '', $code);

				$text = str_replace($match[0], "notextile.. <pre class=\"devutils-highlighted lang-$language\">$code</pre>", $text);
			}
		}

		$parser = new Textile_Parser();
		$html   = $parser->TextileThis($text);

		return $html;
	}

	public function prepareForSally($html) {
		$html = preg_replace('#<p( .*?)?>#', '<p$2>', $html);
		$html = preg_replace('#<pre>#', '<pre style="font-family:monospace;margin:0 10px 5px;padding:3px;border:1px solid #4D4646;line-height:16px">', $html);
		$html = preg_replace('#<h1(.*?)>(.*?)</h1>#', '', $html);
		$html = preg_replace('#<h3(.*?)>#', '<h4$1>', $html);
		$html = preg_replace('#<h2(.*?)>#', '<h3$1>', $html);
		$html = preg_replace('#<ul(.*?)>#', '<ul$1>', $html);
		$html = preg_replace('#<ol(.*?)>#', '<ol$1>', $html);
		$html = preg_replace('#<code(.*?)>#', '<code style="font-family:monospace"$1>', $html);
		$html = preg_replace('#<em(.*?)>#', '<em style="font-style:italic"$1>', $html);
		$html = preg_replace('#<blockquote(.*?)>#', '<blockquote style="font-style:italic;margin:15px 10px 15px 20px"$1>', $html);
		$html = preg_replace('#</h3>#', '</h4>', $html);
		$html = preg_replace('#</h2>#', '</h3>', $html);
		$html = preg_replace('#</h1>#', '</h2>', $html);

		return $html;
	}
}
