<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class WV_Filter_Value implements WV_Filter_Interface {
	protected $getValue;
	protected $filterValue;
	protected $comparator;

	/**
	 *
	 * @param mixed   $filterValue  the value to match wih comparator
	 * @param mixed   $comparator   Closure or '=', '<>', '!=', '<=', '>=', '<', '>', 'LIKE', 'NOT LIKE', 'REGEXP', 'IN'
	 * @param Closure $getValue     a Closure to get the value from an object, by default the value itself will be returned
	 * @return type
	 * @throws WV_Exception_Filter
	 */
	public function __construct($filterValue, $comparator, Closure $getValue = null) {
		if (!is_callable($comparator)) {
			$comparator = $this->getComparatorFromString($comparator);
		}

		if ($getValue === null) {
			$getValue = function($value) {
				return $value;
			};
		}

		$this->comparator  = $comparator;
		$this->filterValue = $filterValue;
		$this->getValue    = $getValue;
	}

	public function isValueFiltered($value) {
		$getValue   = $this->getValue;
		$comparator = $this->comparator;

		return !$comparator($getValue($value), $this->filterValue);
	}

	protected function getComparatorFromString($comparator) {
		switch ($comparator) {
			case '=':
				$comparator = function($value, $filterValue) {
					return $value === $filterValue;
				};
				break;

			case '<>':
				$comparator = function($value, $filterValue) {
					return $value < $filterValue && $value > $filterValue;
				};
				break;

			case '!=':
				$comparator = function($value, $filterValue) {
					return $value !== $filterValue;
				};
				break;

			case '<=':
				$comparator = function($value, $filterValue) {
					return $value <= $filterValue;
				};
				break;

			case '>=':
				$comparator = function($value, $filterValue) {
					return $value >= $filterValue;
				};
				break;

			case '<':
				$comparator = function($value, $filterValue) {
					return $value < $filterValue;
				};
				break;

			case '>':
				$comparator = function($value, $filterValue) {
					return $value > $filterValue;
				};
				break;

			case 'LIKE':
				$comparator = function($value, $filterValue) {
					$filterValue = '@'.str_replace(array('@', '?', '%'), array('\@', '.', '.*'), $filterValue).'@';
					return preg_match($filterValue, $value);
				};
				break;

			case 'NOT LIKE':
				$comparator = function($value, $filterValue) {
					$filterValue = '@'.str_replace(array('@', '?', '%'), array('\@', '.', '.*'), $filterValue).'@';
					return !preg_match($filterValue, $value);
				};
				break;

			case 'REGEXP':
				$comparator = function($value, $filterValue) {
					// check for a delimiter
					if (substr($filterValue, 0, 1) == substr($filterValue, -1)) {
						// use @ as delimiter and excape usages of it
						$filterValue = '@'.str_replace($filterValue, '@', '\@').'@';
					}

					return preg_match($filterValue, $value);
				};
				break;

			case 'IN':
				$comparator = function($value, $filterValue) {
					$value       = sly_makeArray($value);
					$filterValue = sly_makeArray($filterValue);

					foreach ($value as $val) {
						if (!in_array($val, $filterValue)) {
							return false;
						}
					}

					return true;
				};
				break;

			default:
				throw new WV_Exception_Filter(t('devutils_filters_base_unknown', $comparator));
		}

		return $comparator;
	}
}
