<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use wv\BabelCache\Util;

abstract class WV8_Settings {
	/**
	 * Zugriff auf eine Einstellung
	 *
	 * Diese Methode liefert den aktuellen Wert eier bestimmten Einstellung
	 * zurück. Wird die Einstellung nicht gefunden, so wird null zurückgegeben.
	 *
	 * @param  string $namespace  der Namespace der Einstellung
	 * @param  string $name       interner Name der Einstellung
	 * @param  mixed  $default    Standardwert, falls der Wert der Setting leer ist
	 * @param  int    $clang      die gewünschte Sprache der Einstellung
	 * @param  bool   $raw        return serialized data
	 * @return mixed              kann beliebigen Typs sein (je nach Datentyp und dessen Parameter)
	 */
	public static function getValue($namespace, $name, $default = null, $clang = null, $raw = false) {
		$container = sly_Core::getContainer();
		$cache     = $container->getCache();

		// try to read raw value from cache
		$ns      = 'globalsettings.values';
		$key     = md5(md5($namespace).md5($name));
		$setting = $cache->get($ns, $key, null);

		// if no setting has yet been cached, find it and pull the value
		if ($setting === null) {
			$provider = $container['wv8-provider'];
			$sql      = $container['sly-persistence'];
			$obj      = $provider->getSetting($namespace, $name);
			$table    = $obj->getTableName();
			$values   = $sql->magicFetch($table, '`values`', compact('namespace', 'name'));

			if ($values !== false) {
				$values = empty($values) ? array() : WV8_Util::decodeValues($values);
			}

			// broken database or missing config element?
			if ($values === false) {
				throw new WV8_Exception('The database is missing the item '.$namespace.'/'.$name.'. Something went wrong.');
			}

			// store data in cache
			$setting = array('params' => $obj->getParams(), 'datatype' => $obj->getDatatype(), 'raw' => $values);
			$cache->set($ns, $key, $setting);
		}

		// fix missing clangs if needed
		$clangID = WV_Sally::clang($clang);

		if ($clang === WV_Sally::CLANG_ALL || !array_key_exists($clangID, $setting['raw'])) {
			$rawValues = WV8_Util::fixValues($setting['raw']);

			// re-cache data if changed
			if ($rawValues !== $setting['raw']) {
				$setting['raw'] = $rawValues;
				$cache->set($ns, $key, $setting);
			}
		}

		// raw data is required for forms
		if ($raw) {
			return $setting['raw'];
		}

		// prepare deserialization
		$datatype = WV_Datatype::factory($setting['datatype']);
		$params   = $setting['params'];

		// return values for all languages
		if ($clang === WV_Sally::CLANG_ALL) {
			foreach ($setting['raw'] as $clangID => $raw) {
				$value = $datatype->deserializeValue($raw, $params);

				if ($default !== null && $datatype->isValueEmpty($value, $params)) {
					$value = $default;
				}

				$setting['raw'][$clangID] = $value;
			}

			return $setting['raw'];
		}

		if (!array_key_exists($clangID, $setting['raw'])) {
			throw new Exception('Invalid language '.$clangID.' given!');
		}

		$value = $datatype->deserializeValue($setting['raw'][$clangID], $params);

		if ($default !== null && $datatype->isValueEmpty($value, $params)) {
			$value = $default;
		}

		return $value;
	}

	/**
	 * Zugriff auf einen Teilbaum aller Einstellungen
	 *
	 * Diese Methode liefert die aktuellen Werte der Einstellungen
	 * zurück, die im Teilbaum des Namespaces liegen.
	 *
	 * @param  string $namespace  der Namespace der Einstellung
	 * @param  int    $clang      die gewünschte Sprache der Einstellung
	 * @return array              Liste aller gefundenen Einstellungen und ihrer Werte
	 */
	public static function getValues($namespace, $clang = null) {
		$container = sly_Core::getContainer();
		$provider  = $container['wv8-provider'];
		$all       = $provider->getAllSettings();
		$result    = array();

		foreach ($all as $ns => $settings) {
			if ($ns !== $namespace && !sly_Util_String::startsWith($ns, "$namespace.")) continue;

			foreach ($settings as $name => $setting) {
				$key   = trim(substr($ns, strlen($namespace)), '.');
				$key   = empty($key) ? $name : $key.'.'.$name;
				$value = self::getValue($ns, $name, null, $clang);

				$result[$key] = $value;
			}
		}

		return $result;
	}
}
